/*
    decoder.cpp
    Decoder for the Latent Words Language Model.
    $Id: decoder.cpp,v 1.7 2010/03/19 10:16:54 daichi Exp $

*/
#include "lwlm.h"
#include "util.h"
#include "io.h"
#include <cstdio>
#include <cstring>
#include <unistd.h>

static char *revision = "$Id: decoder.cpp,v 1.7 2010/03/19 10:16:54 daichi Exp $";
static char *options = "o:e:f:K:B:N:I:h";
static char *format = "%s (%.03f)  ";

void
usage ()
{
	printf("lwlm-decoder, The Latent Words Languge Model decoder.\n");
	printf("%s\n", revision);
	printf("Copyright (C) 2010 Daichi Mochihashi, All rights reserved.\n");
	printf("usage: lwlm-decode OPTIONS test model\n"
	       "OPTIONS:\n"
	       "-B [b]    burn-in iterations (default 100)\n"
	       "-N [n]    number of iterations to collect samples (default 100)\n"
	       "-I [i]    interval to collect posterior samples (default 1)\n"
	       "-K [K]    number of possible candidates (default 5)\n"
	       "-e [eps]  threshold for candidate probability (default 0)\n"
	       "-f [fmt]  printf format of candidates (default \"%s\")\n"
	       "-o [file] filename for output\n"
	       "-h        this help\n", format);
	exit(0);
}

int
main (int argc, char *argv[])
{
	LWLM *lwlm;
	char *test, *model, *out = NULL;
	int c, order, iter = 100, burn = 100, interval = 1, K = 5;
	double alpha, epsilon = 0;

	while ((c = getopt(argc, argv, options)) != -1)
	{
		switch (c) {
		case 'I' : interval = atoi(optarg); break;
		case 'e' : epsilon = atof(optarg); break;
		case 'f' : format = strdup(optarg); break;
		case 'N' : iter = atoi(optarg); break;
		case 'B' : burn = atoi(optarg); break;
		case 'o' : out = strdup(optarg); break;
		case 'K' : K = atoi(optarg); epsilon = 0; break;
		case 'h' : usage (); break;
		default  : usage (); break;
		}
	}
	
	if (argc - optind < 2)
		usage ();
	else {
		test  = argv[optind];
		model = argv[optind + 1];
	}

	if (lwlm_load_param (model, &order, &alpha) == true)
	{
		lwlm = new LWLM (order, alpha, iter);
	}
	else {
		error("can't load model parameters.\n");
		exit(1);
	}

	lwlm->load (model);
	lwlm->read (test);
	lwlm->decode (out, format, burn, interval, epsilon, K);

}

