/*
    LWLM.h
    the Latent Words Language Model.
    $Id: lwlm.h,v 1.9 2010/03/18 08:17:29 daichi Exp $

*/
#ifndef _LWLM_H_
#define _LWLM_H_
#include "hpylm.h"
#include "table.h"
#include <string>
#include <vector>
#include <ext/hash_map>
#define  cons make_pair
#define  push_front(v,x)  (v).insert((v).begin(),x)

using namespace std;
using namespace __gnu_cxx;

typedef hash_map <int, string> id_hash;
typedef hash_map <string, int> lex_hash;
typedef vector<vector<int> *>::iterator siterator;

namespace __gnu_cxx {
	template <> struct hash<string>
	{
		size_t operator() (const string &s) const
		{
			string::const_iterator it;
			size_t result = 0;
			for (it = s.begin(); it != s.end(); it++)
				result = (result * 131) + *it;
			return result;
		}
	};
}

struct Word {
	witerator observed;
	witerator hidden;
	int width;
	Word (witerator ot, witerator ht, int n) {
		observed = ot;
		hidden = ht;
		width = n;
	};
};

struct LWLM {
	double alpha;
	int order;
	int iter;
	int L;

	HPYLM *lm;
	TransTable *table;

	id_hash rdict;
	lex_hash dict;
	vector<Word *> words;
	vector<vector<int> *> hidden;
	vector<vector<int> *> observed;

	/* parse */
	bool read (char *file);
	bool parse (vector<int> *s);

	/* inference */
	bool learn ();
	int  gibbs_draw (int *hidden, int *observed, int width);
	int  slice_gibbs_draw (int *hidden, int *observed, int width);
	int  gibbs_draw (witerator hidden, witerator observed, int width);
	int  slice_gibbs_draw (witerator hidden, witerator observed, int width);
	bool slice_sample_agenda (vector<int> *agenda, witerator word, witerator history);
	bool hpylm_slice_agenda (ngram *node, double slice, vector<int> *agenda);

	/* decoding */
	void decode (char *out, char *fmt, int burn, int interval, double epsilon, int K);
	void gibbs_decode (vector<vector<int> *> &sampled, vector<int> &s, int burn, int interval);
	void write_decoded (vector<vector<int> *> &sampled, vector<int> &s,
			    FILE *fp, char *fmt, double epsilon, int K);

	/* probability */
	double perplexity ();

	/* I/O */
	void save (char *file);
	void save_param (char *file);
	void save_latent (char *file);
	void save_lexicon (char *file);
	void load (char *file);
	void load_lexicon (char *file);

	/* dictionary */
	char *ids (int id);
	int sid (char *s);

	/* print */
	void print ();

	/* const/destructors */
	LWLM (int order_, double alpha_, int iter_=1);
};



#endif
