/*
    LWLM.cpp
    The Latent Words Language Model.
    $Id: main.cpp,v 1.11 2010/03/19 10:16:54 daichi Exp $

*/
#include "lwlm.h"
#include "log.h"
#include "util.h"
#include <cstdio>
#include <cstdlib>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>

static char revision[] = "$Id: main.cpp,v 1.11 2010/03/19 10:16:54 daichi Exp $";
static char options[] = "a:n:N:h";

void
usage ()
{
	printf("lwlm, The Latent Words Languge Model.\n");
	printf("%s\n", revision);
	printf("Copyright (C) 2010 Daichi Mochihashi, All rights reserved.\n");
	printf("usage: lwlm OPTIONS train model\n");
	printf("OPTIONS\n"
	       "-n  n-gram order (default 3, trigram)\n"
	       "-a  Dirichlet hyperparameter of translation table (default 0.01)\n"
	       "-N  number of Gibbs iterations\n"
	       "-h  help\n");
	exit(0);
}

void
prepare_output (char *model)
{
	mode_t mode =  S_IRUSR | S_IWUSR | S_IXUSR |
		       S_IRGRP | S_IXGRP |
		       S_IROTH | S_IXOTH;

	if (opendir(model) != NULL) /* model exists */
	{
		error("model directory %s already exists.\n", model);
		exit(1);
	}
	if (mkdir(model, mode) != 0 ||
	    mkdir(strconcat(model, "/lm"), mode) != 0)
	{
		error("can't create model directory %s.\n", model);
		exit(1);
	}
}

int
main (int argc, char *argv[])
{
	LWLM *lwlm;
	char *train, *model;
	int c, order = 2, iter = 1;
	double alpha = 0.01;

	while ((c = getopt(argc, argv, options)) != -1)
	{
		switch (c) {
		case 'a' : alpha = atof(optarg); break;
		case 'n' : order = atoi(optarg) - 1; break;   /* Markov order */
		case 'N' : iter = atoi(optarg); break;
		case 'h' : usage (); break;
		default  : usage (); break;
		}
	}
	if (argc - optind < 2)
		usage ();
	else {
		train = argv[optind];
		model = argv[optind + 1];
	}
	
	prepare_output (model);
	logging start (argv, model);
	
	lwlm = new LWLM (order, alpha, iter);

	lwlm->read (train);
	lwlm->learn ();
	lwlm->save (model);

	return 0;
}


