/*
    save.cpp
    save HPYLM to files.
    $Id: save.cpp,v 1.3 2010/03/18 07:28:41 daichi Exp $

*/
#include <cstdio>
#include <cstdlib>
#include <ext/hash_map>
#include <utility>
#include "hpylm.h"
#include "lwlm.h"
#include "hash.h"
#include "util.h"
#include "io.h"

using namespace std;
using namespace __gnu_cxx;

typedef struct {
	int id;
	FILE *fp;
	node_map *hash;
} fphash;

/*
 *  utility functions
 *
 */

int
hpylm_write_restaurant (restaurant *rest, fphash *arg)
{
	FILE *fp = arg->fp;
	vector<int>::iterator it;
	
	fprintf(fp, "%d\t%d", rest->word, rest->ncounts);
	for (it = rest->tables.begin(); it != rest->tables.end(); it++)
		fprintf(fp, "\t%d", *it);
	fprintf(fp, "\n");
	
	return 0;
}

node_map *
hpylm_write_nodes (ngram *node, fphash *arg)
{
	int i;
	FILE *fp = arg->fp;
	node_map *hash = arg->hash;
	record *records = node->records;

	/* register */
	hash->insert (cons(node, arg->id++));
	
	/* node */
	fprintf(fp, "%d\t%d\t%d\t%d\n",
		node->level,
		node->word,
		node->ncounts,
		node->ntables);
	/* bounds */
	for (i = 0; i < records->bounds.size(); i++)
		fprintf(fp, "%d%s", records->bounds[i],
			(i == records->bounds.size() - 1) ? "\n" : "\t");
	/* restaurants */
	records->foreach ((recfunc)hpylm_write_restaurant, arg);
	fprintf(fp, "\n");

	/* write recursively */
	hash_foreach (node->children, (nodefunc)hpylm_write_nodes, arg);

	return hash;
}

node_map *
hpylm_save_nodes (ngram *tree, char *file)
{
	node_map *hash = new node_map;
	fphash arg;
	FILE *fp;

	if ((fp = fopen(strconcat(file, "/nodes"), "w")) == NULL)
	{
		perror("fopen");
		return NULL;
	}
	arg.id = 0;
	arg.fp = fp;
	arg.hash = hash;
	
	if ((hash = hpylm_write_nodes(tree, &arg)) == NULL)
	{
		error("hpylm_save_nodes: can't write nodes.\n");
		return NULL;
	}
	
	fclose(fp);
	return hash;
}

void
hpylm_write_child (ngram *node, fphash *arg)
{
	fprintf(arg->fp, "\t%d\n", (*arg->hash)[node]);
}

void
hpylm_write_tree (ngram *node, fphash *arg)
{
	node_map *nodemap = arg->hash;
	FILE *fp = arg->fp;

	if (!node->children->empty())
	{
		fprintf(fp, "%d\n", (*nodemap)[node]);
		hash_foreach (node->children, (nodefunc) hpylm_write_child, arg);
		fprintf(fp, "\n");
	}
	hash_foreach (node->children, (nodefunc) hpylm_write_tree, arg);
}

void
hpylm_save_tree (ngram *tree, char *file, node_map *nodemap)
{
	fphash arg;
	FILE *fp;

	if ((fp = fopen(strconcat(file, "/tree"), "w")) == NULL)
	{
		perror("fopen");
		return;
	}
	arg.fp = fp;
	arg.hash = nodemap;
	hpylm_write_tree (tree, &arg);
	fclose(fp);
}

void
hpylm_save_param (HPYLM *lm, char *file)
{
	FILE *fp;
	int i;

	if ((fp = fopen(strconcat(file, "/param"), "w")) == NULL)
	{
		perror("fopen");
		return;
	}
	fprintf(fp, "%d\t%d\n", lm->order, lm->lexicon);
	for (i = 0; i < lm->d.size(); i++)
	{
		fprintf(fp, "%.7e\t%.7e\n", lm->d[i], lm->theta[i]);
	}
	fclose(fp);
}

/*
 *  member functions
 *
 */

void
HPYLM::save (char *file)
{
	node_map *nodemap;
	
	if ((nodemap = hpylm_save_nodes (tree, file)) == NULL)
	{
		error("HPYLM::save: can't save nodes.\n");
		return;
	}
	hpylm_save_tree (tree, file, nodemap);
	hpylm_save_param (this, file);

	delete nodemap;
}

void
LWLM::save_lexicon (char *file)
{
	FILE *fp;
	int v;

	if ((fp = fopen(strconcat(file, "/lexicon"), "w")) == NULL)
	{
		perror("fopen");
		return;
	}
	for (v = 0; v < L; v++)
		fprintf(fp, "%s\n", ids(v));
	
	fclose(fp);
}

void
LWLM::save_latent (char *file)
{
	FILE *fp;
	siterator h;
	witerator w;

	if ((fp = fopen(strconcat(file, "/latent"), "w")) == NULL)
	{
		perror("fopen");
		return;
	}
	for (h = hidden.begin(); h != hidden.end(); h++)
	{
		for (w = (*h)->begin(); w != (*h)->end(); w++)
			if (*w != EOS) fprintf(fp, "%s ", ids(*w));
		fprintf(fp, "\n");
	}

	fclose(fp);
}

void
LWLM::save_param (char *file)
{
	FILE *fp;

	if ((fp = fopen(strconcat(file, "/param"), "w")) == NULL)
	{
		perror("fopen");
		return;
	}
	fprintf(fp, "%d\t%.7e\n", order, alpha);
	fclose(fp);
}

void
LWLM::save (char *file)
{
	eprintf("saving model.. ");
	save_param (file);
	save_lexicon (file);
	save_latent (file);
	table->save (file, this);
	lm->save (strconcat(file, "/lm"));
	eprintf("done.\n");
}

