/*
    util.cpp
    utility functions in C++.
    $Id: util.cpp,v 1.4 2010/03/17 16:41:57 daichi Exp $
*/
#include <cstdio>
#include <cstdarg>
#include <cstring>
#include <ctype.h>
#include <cmath>
#include <string>
#include "util.h"
#include "debug.h"
extern "C" {
#include <sys/time.h>
}
#define  min(a,b)	((a) < (b) ? (a) : (b))
#define  max(a,b)	((a) > (b) ? (a) : (b))

using namespace std;

int myclock ()
{
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return tv.tv_sec;
}

char *
rtime (double t)
{
	int hour, min, sec;
	static char buf[BUFSIZ];

	hour = (int)floor((int)t / 60 / 60);
	min  = (int)floor(((int)t % (60 * 60)) / 60);
	sec  = (int)floor((int)t % 60);
	sprintf(buf, "%2d:%02d:%02d", hour, min, sec);
	
	return buf;
}

int
vmax (double *v, int n)
{
	int i, m = 0;
	for (i = 0; i < n; i++)
		if (v[i] > v[m])
			m = i;
	return m;
}

void
vec_normalize (double *v, int n)
{
	double z = 0;
	int i;

	for (i = 0; i < n; i++)
		z += v[i];
	for (i = 0; i < n; i++)
		if (v[i] > 0)
			v[i] /= z;
}

double
logsumexp (double x, double y, int init)
{
	double vmin = min(x, y);
	double vmax = max(x, y);
	
	if (init) return y;
	if (x == y) return x + log(2.0);
	if (vmax > vmin + 50)
		return vmax;
	else
		return vmax + log(1 + exp(vmin - vmax));
}

bool
isspaces (const char *s)
{
	for (; *s; s++)
		if (!isspace(*s))
			return false;
	return true;
}
	
bool
isspaces (const string &line)
{
	string::const_iterator it;

	for (it = line.begin(); it != line.end(); it++)
		if (!isspace(*it))
			return false;
	return true;
}

char *
strconcat (const char *s, const char *t)
{
	static char z[BUFSIZ];
	strcpy(z, s);
	strcat(z, t);
	return strdup(z);
}

bool
converged (double u, double v, double threshold)
{
	/* return true if |u - v|/|u| < threshold */
	if (fabs((u - v) / v) < threshold)
		return true;
	else
		return false;
}

int
eprintf (const char *fmt, ...)
{
	int n;
	va_list ap;

	va_start(ap, fmt);
	n = vfprintf(stderr, fmt, ap);
	va_end(ap);
	fflush(stderr);

	return n;
}
	
int
error (const char *fmt, ...)
{
	int n;
	va_list ap;
	
	va_start(ap, fmt);
	n = vfprintf(stderr, fmt, ap);
	va_end(ap);
	return n;
}

#if 0

int
main (int argc, char *argv[])
{
	char *line = argv[1];

	if (isspaces(line))
		puts("SPACES!");
	else
		puts("NO SPACES!");

}

#endif
